@@ -16,7 +16,7 @@ from poetry.console.commands.installer_command import InstallerCommand
 
 class AddCommand(InstallerCommand, InitCommand):
     name = "add"
-    description = "Adds a new dependency to <comment>pyproject.toml</>."
+    description = "Adds a new dependency to <comment>pyproject.toml</> and installs it."
 
     arguments = [argument("name", "The packages to add.", multiple=True)]
     options = [
@@ -30,10 +30,8 @@ class AddCommand(InstallerCommand, InitCommand):
         option(
             "dev",
             "D",
-            (
-                "Add as a development dependency. (<warning>Deprecated</warning>) Use"
-                " --group=dev instead."
-            ),
+            "Add as a development dependency. (<warning>Deprecated</warning>) Use"
+            " --group=dev instead.",
         ),
         option("editable", "e", "Add vcs/path dependencies as editable."),
         option(
@@ -66,10 +64,8 @@ class AddCommand(InstallerCommand, InitCommand):
         option(
             "dry-run",
             None,
-            (
-                "Output the operations but do not execute anything (implicitly enables"
-                " --verbose)."
-            ),
+            "Output the operations but do not execute anything (implicitly enables"
+            " --verbose).",
         ),
         option("lock", None, "Do not perform operations (only update the lockfile)."),
     ]
@@ -83,6 +79,8 @@ You can specify a package in the following forms:
   - A git url (<b>git+https://github.com/python-poetry/poetry.git</b>)
   - A git url with a revision\
  (<b>git+https://github.com/python-poetry/poetry.git#develop</b>)
+  - A subdirectory of a git repository\
+ (<b>git+https://github.com/python-poetry/poetry.git#subdirectory=tests/fixtures/sample_project</b>)
   - A git SSH url (<b>git+ssh://github.com/python-poetry/poetry.git</b>)
   - A git SSH url with a revision\
  (<b>git+ssh://github.com/python-poetry/poetry.git#develop</b>)
@@ -126,7 +124,9 @@ The add command adds required packages to your <comment>pyproject.toml</> and in
         # dictionary.
         content: dict[str, Any] = self.poetry.file.read()
         poetry_content = content["tool"]["poetry"]
-        project_name = canonicalize_name(poetry_content["name"])
+        project_name = (
+            canonicalize_name(name) if (name := poetry_content.get("name")) else None
+        )
 
         if group == MAIN_GROUP:
             if "dependencies" not in poetry_content:
@@ -302,5 +302,5 @@ The add command adds required packages to your <comment>pyproject.toml</> and in
             " be skipped:\n"
         )
         for name in existing_packages:
-            self.line(f"  • <c1>{name}</c1>")
+            self.line(f"  - <c1>{name}</c1>")
         self.line(self._hint_update_packages)
